@@ -11,31 +11,30 @@
 """
 from __future__ import absolute_import
 
+import codecs
+import doctest
 import re
 import sys
 import time
-import codecs
-import platform
 from os import path
-import doctest
-
-from six import itervalues, StringIO, binary_type, text_type, PY2
-from distutils.version import LooseVersion
+from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
+from packaging.specifiers import SpecifierSet, InvalidSpecifier
+from packaging.version import Version
+from six import itervalues, StringIO, binary_type, text_type, PY2
 
 import sphinx
 from sphinx.builders import Builder
-from sphinx.util import force_decode, logging
-from sphinx.util.nodes import set_source_info
-from sphinx.util.console import bold  # type: ignore
-from sphinx.util.osutil import fs_encoding
 from sphinx.locale import _
+from sphinx.util import force_decode, logging
+from sphinx.util.console import bold  # type: ignore
+from sphinx.util.nodes import set_source_info
+from sphinx.util.osutil import fs_encoding
 
-if False:
-    # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterable, List, Sequence, Set, Tuple  # NOQA
+if TYPE_CHECKING:
+    from typing import Any, Callable, Dict, IO, Iterable, List, Optional, Sequence, Set, Tuple  # NOQA
     from sphinx.application import Sphinx  # NOQA
 
 logger = logging.getLogger(__name__)
@@ -57,28 +56,23 @@ else:
         return text
 
 
-def compare_version(ver1, ver2, operand):
-    # type: (unicode, unicode, unicode) -> bool
-    """Compare `ver1` to `ver2`, relying on `operand`.
+def is_allowed_version(spec, version):
+    # type: (unicode, unicode) -> bool
+    """Check `spec` satisfies `version` or not.
+
+    This obeys PEP-440 specifiers:
+    https://www.python.org/dev/peps/pep-0440/#version-specifiers
 
     Some examples:
 
-        >>> compare_version('3.3', '3.5', '<=')
+        >>> is_allowed_version('3.3', '<=3.5')
         True
-        >>> compare_version('3.3', '3.2', '<=')
+        >>> is_allowed_version('3.3', '<=3.2')
         False
-        >>> compare_version('3.3a0', '3.3', '<=')
+        >>> is_allowed_version('3.3', '>3.2, <4.0')
         True
     """
-    if operand not in ('<=', '<', '==', '>=', '>'):
-        raise ValueError("'%s' is not a valid operand.")
-    v1 = LooseVersion(ver1)
-    v2 = LooseVersion(ver2)
-    return ((operand == '<=' and (v1 <= v2)) or
-            (operand == '<' and (v1 < v2)) or
-            (operand == '==' and (v1 == v2)) or
-            (operand == '>=' and (v1 >= v2)) or
-            (operand == '>' and (v1 > v2)))
+    return Version(version) in SpecifierSet(spec)
 
 
 # set up the necessary directives
@@ -120,7 +114,11 @@ class TestDirective(Directive):
         if test is not None:
             # only save if it differs from code
             node['test'] = test
-        if self.name == 'testoutput':
+        if self.name == 'doctest':
+            node['language'] = 'pycon'
+        elif self.name == 'testcode':
+            node['language'] = 'python'
+        elif self.name == 'testoutput':
             # don't try to highlight output
             node['language'] = 'none'
         node['options'] = {}
@@ -143,16 +141,14 @@ class TestDirective(Directive):
                 node['options'][flag] = (option[0] == '+')
         if self.name == 'doctest' and 'pyversion' in self.options:
             try:
-                option = self.options['pyversion']
-                # :pyversion: >= 3.6   -->   operand='>=', option_version='3.6'
-                operand, option_version = [item.strip() for item in option.split()]
-                running_version = platform.python_version()
-                if not compare_version(running_version, option_version, operand):
+                spec = self.options['pyversion']
+                python_version = '.'.join(str(v) for v in sys.version_info[:3])
+                if not is_allowed_version(spec, python_version):
                     flag = doctest.OPTIONFLAGS_BY_NAME['SKIP']
                     node['options'][flag] = True  # Skip the test
-            except ValueError:
+            except InvalidSpecifier:
                 self.state.document.reporter.warning(
-                    _("'%s' is not a valid pyversion option") % option,
+                    _("'%s' is not a valid pyversion option") % spec,
                     line=self.lineno)
         return [node]
 
@@ -227,17 +223,18 @@ class TestGroup(object):
 
 
 class TestCode(object):
-    def __init__(self, code, type, lineno, options=None):
-        # type: (unicode, unicode, int, Dict) -> None
+    def __init__(self, code, type, filename, lineno, options=None):
+        # type: (unicode, unicode, Optional[str], int, Optional[Dict]) -> None
         self.code = code
         self.type = type
+        self.filename = filename
         self.lineno = lineno
         self.options = options or {}
 
     def __repr__(self):  # type: ignore
         # type: () -> unicode
-        return 'TestCode(%r, %r, %r, options=%r)' % (
-            self.code, self.type, self.lineno, self.options)
+        return 'TestCode(%r, %r, filename=%r, lineno=%r, options=%r)' % (
+            self.code, self.type, self.filename, self.lineno, self.options)
 
 
 class SphinxDocTestRunner(doctest.DocTestRunner):
@@ -278,6 +275,8 @@ class DocTestBuilder(Builder):
     Runs test snippets in the documentation.
     """
     name = 'doctest'
+    epilog = ('Testing of doctests in the sources finished, look at the '
+              'results in %(outdir)s/output.txt.')
 
     def init(self):
         # type: () -> None
@@ -368,6 +367,36 @@ Doctest summary
             doctree = self.env.get_doctree(docname)
             self.test_doc(docname, doctree)
 
+    def get_filename_for_node(self, node, docname):
+        # type: (nodes.Node, unicode) -> str
+        """Try to get the file which actually contains the doctest, not the
+        filename of the document it's included in."""
+        try:
+            filename = path.relpath(node.source, self.env.srcdir)\
+                .rsplit(':docstring of ', maxsplit=1)[0]
+        except Exception:
+            filename = self.env.doc2path(docname, base=None)
+        if PY2:
+            return filename.encode(fs_encoding)
+        return filename
+
+    @staticmethod
+    def get_line_number(node):
+        # type: (nodes.Node) -> Optional[int]
+        """Get the real line number or admit we don't know."""
+        # TODO:  Work out how to store or calculate real (file-relative)
+        #       line numbers for doctest blocks in docstrings.
+        if ':docstring of ' in path.basename(node.source or ''):
+            # The line number is given relative to the stripped docstring,
+            # not the file.  This is correct where it is set, in
+            # `docutils.nodes.Node.setup_child`, but Sphinx should report
+            # relative to the file, not the docstring.
+            return None
+        if node.line is not None:
+            # TODO: find the root cause of this off by one error.
+            return node.line - 1
+        return None
+
     def test_doc(self, docname, doctree):
         # type: (unicode, nodes.Node) -> None
         groups = {}  # type: Dict[unicode, TestGroup]
@@ -394,13 +423,16 @@ Doctest summary
                 return isinstance(node, (nodes.literal_block, nodes.comment)) \
                     and 'testnodetype' in node
         for node in doctree.traverse(condition):
-            source = 'test' in node and node['test'] or node.astext()
+            source = node['test'] if 'test' in node else node.astext()
+            filename = self.get_filename_for_node(node, docname)
+            line_number = self.get_line_number(node)
             if not source:
                 logger.warning('no code/output in %s block at %s:%s',
                                node.get('testnodetype', 'doctest'),
-                               self.env.doc2path(docname), node.line)
+                               filename, line_number)
             code = TestCode(source, type=node.get('testnodetype', 'doctest'),
-                            lineno=node.line, options=node.get('options'))
+                            filename=filename, lineno=line_number,
+                            options=node.get('options'))
             node_groups = node.get('groups', ['default'])
             if '*' in node_groups:
                 add_to_all_groups.append(code)
@@ -414,12 +446,12 @@ Doctest summary
                 group.add_code(code)
         if self.config.doctest_global_setup:
             code = TestCode(self.config.doctest_global_setup,
-                            'testsetup', lineno=0)
+                            'testsetup', filename=None, lineno=0)
             for group in itervalues(groups):
                 group.add_code(code, prepend=True)
         if self.config.doctest_global_cleanup:
             code = TestCode(self.config.doctest_global_cleanup,
-                            'testcleanup', lineno=0)
+                            'testcleanup', filename=None, lineno=0)
             for group in itervalues(groups):
                 group.add_code(code)
         if not groups:
@@ -428,7 +460,7 @@ Doctest summary
         self._out('\nDocument: %s\n----------%s\n' %
                   (docname, '-' * len(docname)))
         for group in itervalues(groups):
-            self.test_group(group, self.env.doc2path(docname, base=None))
+            self.test_group(group)
         # Separately count results from setup code
         res_f, res_t = self.setup_runner.summarize(self._out, verbose=False)
         self.setup_failures += res_f
@@ -447,13 +479,8 @@ Doctest summary
         # type: (unicode, unicode, unicode, Any, bool) -> Any
         return compile(code, name, self.type, flags, dont_inherit)
 
-    def test_group(self, group, filename):
-        # type: (TestGroup, unicode) -> None
-        if PY2:
-            filename_str = filename.encode(fs_encoding)
-        else:
-            filename_str = filename
-
+    def test_group(self, group):
+        # type: (TestGroup) -> None
         ns = {}  # type: Dict
 
         def run_setup_cleanup(runner, testcodes, what):
@@ -468,7 +495,7 @@ Doctest summary
             # simulate a doctest with the code
             sim_doctest = doctest.DocTest(examples, {},
                                           '%s (%s code)' % (group.name, what),
-                                          filename_str, 0, None)
+                                          testcodes[0].filename, 0, None)
             sim_doctest.globs = ns
             old_f = runner.failures
             self.type = 'exec'  # the snippet may contain multiple statements
@@ -489,10 +516,10 @@ Doctest summary
                 try:
                     test = parser.get_doctest(  # type: ignore
                         doctest_encode(code[0].code, self.env.config.source_encoding), {},  # type: ignore  # NOQA
-                        group.name, filename_str, code[0].lineno)
+                        group.name, code[0].filename, code[0].lineno)
                 except Exception:
                     logger.warning('ignoring invalid doctest code: %r', code[0].code,
-                                   location=(filename, code[0].lineno))
+                                   location=(code[0].filename, code[0].lineno))
                     continue
                 if not test.examples:
                     continue
@@ -520,7 +547,7 @@ Doctest summary
                     lineno=code[0].lineno,
                     options=options)
                 test = doctest.DocTest([example], {}, group.name,  # type: ignore
-                                       filename_str, code[0].lineno, None)
+                                       code[0].filename, code[0].lineno, None)
                 self.type = 'exec'  # multiple statements again
             # DocTest.__init__ copies the globs namespace, which we don't want
             test.globs = ns
