@@ -6,7 +6,7 @@
     Set up everything for use of JSMath to display math in HTML
     via JavaScript.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,18 +14,20 @@ from docutils import nodes
 
 import sphinx
 from sphinx.application import ExtensionError
+from sphinx.ext.mathbase import get_node_equation_number
 from sphinx.ext.mathbase import setup_math as mathbase_setup
+from sphinx.locale import _
 
 
 def html_visit_math(self, node):
-    self.body.append(self.starttag(node, 'span', '', CLASS='math'))
+    self.body.append(self.starttag(node, 'span', '', CLASS='math notranslate'))
     self.body.append(self.encode(node['latex']) + '</span>')
     raise nodes.SkipNode
 
 
 def html_visit_displaymath(self, node):
     if node['nowrap']:
-        self.body.append(self.starttag(node, 'div', CLASS='math'))
+        self.body.append(self.starttag(node, 'div', CLASS='math notranslate'))
         self.body.append(self.encode(node['latex']))
         self.body.append('</div>')
         raise nodes.SkipNode
@@ -34,9 +36,11 @@ def html_visit_displaymath(self, node):
         if i == 0:
             # necessary to e.g. set the id property correctly
             if node['number']:
-                self.body.append('<span class="eqno">(%s)</span>' %
-                                 node['number'])
-            self.body.append(self.starttag(node, 'div', CLASS='math'))
+                number = get_node_equation_number(self, node)
+                self.body.append('<span class="eqno">(%s)' % number)
+                self.add_permalink_ref(node, _('Permalink to this equation'))
+                self.body.append('</span>')
+            self.body.append(self.starttag(node, 'div', CLASS='math notranslate'))
         else:
             # but only once!
             self.body.append('<div class="math">')
